/////////////////////////////////////////////////////////////////////////////////////////////////////
/*このスケッチは愛知県農業総合試験場において制作した、栽培環境モニタリング装置のためのサンプルスケッチです。
本スケッチによる機器の故障をはじめとしたその他トラブルに対して責任を負いません。

このスケッチでは、実際に装置を動作しデータの記録を行います。


測定装置本体について

ESP32DevkitCを使用し、マイクロSDカードリーダー、RTC(DS1307）、mosfetk4017を使用した測定装置本体を使用します。
測定用センサーとして、
・温湿度：AM2302（DHT22）センサー
・温度（地温・水温）：ds18b20センサー（防水プローブ付き）
・土壌水分：METER社　10HSセンサー(静電容量式）
・EC：簡易ECセンサー（自作）
・日射：簡易日射センサー（自作）
を接続し、各データをマイクロSDカードに任意の時間間隔（ディープスリープで設定した時間）で記録します。


*//////////////////////////////////////////////////////////////////////////////////////////////////////

//ディープスリープ設定
#define uS_TO_S_FACTOR 1000000ULL  //ディープスリープの指定はマイクロ秒で扱いにくいので秒単位に変換
#define TIME_TO_SLEEP  10 //スリープ時間を設定（秒単位）
RTC_DATA_ATTR int bootCount = 0; //動作回数をカウントし、初回のみの動作を指定するためにbootCountを用意

//mosfet設定
#define mosfet 16

//SDカード用ライブラリ(ダウンロード不要）
#include "FS.h"
#include "SD.h"
#include "SPI.h"//WROOM
const char* fname1 = "/test1.csv";  //ＳＤカードのルートにtest1.csvを作成
//const char* fname2 = "/test2.txt";  //ＳＤカードのルートにtest2.txtを作成
File f1, f2;  //ファイルｆ１、ｆ２を用意、ｆ２は拡張用
enum { sd_sck = 19,sd_miso = 21,sd_mosi = 5,sd_ss = 17 };
//SDカードの2番ピンにSS17、3番ピンにmosi5、4番ピンに電源＋、5番ピンにSCK19、6番ピンに電源－、7番ピンにmiso21

//RTC用ライブラリ
#include "RTClib.h"//RTC用ライブラリ（ライブラリマネージャーから取得）
#include "Wire.h"//I2C通信用ライブラリ（任意のピンでRTCを起動するため）
RTC_DS1307 rtc;//1307モジュールは５Vを要求

//AM2302(DHT用)ライブラリ
#include "DHT.h"//（ライブラリマネージャーから取得）
#define DHTPIN 25//DHT22の通信用にGPIO25を使用
                 //GPIO25と3.3Vの間に4.7kω抵抗をつなげること（プルアップ）
#define DHTTYPE DHT22
DHT dht(DHTPIN, DHTTYPE);

//ds18b20用ライブラリ
#include <OneWire.h>
#include <DallasTemperature.h>//（ライブラリマネージャーから取得）
#define ONE_WIRE_BUS 26//ds18b20センサーの通信用にGPIO26を使用
                       //GPIO26と3.3Vの間に4.7kω抵抗をつなげること（プルアップ）
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);

//日射用設定
#define SRmVPin 34//簡易日射計として使用するソーラーパネルのプラス端子をGPIO34に接続する。
                  //ソーラーパネルのマイナス端子はブレッドボード上のGNDに配線する
                  //ソーラーパネルのプラス端子を接続したESP32のGPIO34と、GNDの間にまたがるように22ω抵抗を接続する。

//EC用設定
int R1= 510;//分圧用抵抗、ステンレスねじM3×20で3㎝感覚の場合は510ωが適切
int Ra=22;  //本体の内部抵抗値、arduino Ra=25; esp32 Ra=22
int ECPin= 35;    //INPUT 分圧された電圧を読み取る。電極の一方を接続。もう一方はGNDに接続。
int ECPower =32;  //OUTPUT 短時間3.3Vを出力するために必要。デジタルでも問題ない。
                  //ECPinとECPowerとの間にR1（510ω）を接続してR1と測定対象で分圧する
float K=3;//セル定数：電極の距離、面積によって決まる。ECを校正する場合はこの数値を調整する。
float soilEC=0;
float raw= 0;
float Vin= 3.3;  //ESP32の基準電圧が3.3V、Arduinoなら5V
float Vdrop= 0;
float Rc= 0;
float buffer=0;

//土壌水分用設定
#define soilmoistPin 33//土壌水分センサーの通信用にGPIO33を使用する。



void setup() {
bootCount++;  // 起動回数カウントアップ
pinMode(mosfet,OUTPUT);
pinMode(SRmVPin, INPUT);
pinMode(ECPin,INPUT);
pinMode(ECPower,OUTPUT);
  R1=(R1+Ra);//分圧用抵抗と内部抵抗を合計

digitalWrite(mosfet,HIGH);//電源on、これ以降各センサーとSDカード、RTCに電源が入る
delay(1000);//各センサー動作待ち

  Serial.begin(115200);
  dht.begin();
  sensors.begin();
    //Wire は前の数字がSDA、後の数字がSCL
  Wire.begin(22,23);//SDA(22),SCL(23)の順に入力する
  rtc.begin();
//rtc.adjustはPCからインクルードしたときのデータのみが反映される
//サンプルプログラムにはコメントアウトとして残すが、事前にRTC用のスケッチでRTCに現在時刻を登録したほうが良い。
  // rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));//インクルード時のPCの現時刻を使用
  // rtc.adjust(DateTime(2014, 1, 21, 3, 0, 0));//任意の時刻を使用
  float h = dht.readHumidity();
  float t = dht.readTemperature();

  sensors.requestTemperatures(); // Send the command to get temperatures
  float soiltemp = sensors.getTempCByIndex(0);

  int soilmoist = analogRead(soilmoistPin);

  int SRmV = analogRead(SRmVPin); 

  digitalWrite(ECPower,HIGH);
  raw= analogRead(ECPin);
  delay(2);//esp32の計算速度が速いので2ミリ秒待機:長いほど出力値は小さくなるが安定？
  raw= analogRead(ECPin);//静電容量の影響を抑えて再度計測
  digitalWrite(ECPower,LOW);
  Vdrop=(Vin*raw)/4096.0; //arduinoでは4096→1024に変更する。
  Rc=(Vdrop*R1)/(Vin-Vdrop);
  Rc=Rc-Ra; //acounting for Digital Pin Resitance
  float soilEC = 1000/(Rc*K);//セル定数が大きくなるほど分母が大きくなるのでECの出力値は下がる。


  SPI.end();                                   //SPI通信を一度初期化
  SPI.begin(sd_sck, sd_miso, sd_mosi, sd_ss);  //指定したSPIのピンで通信を開始
  SD.begin(sd_ss, SPI, 24000000, "/sd");       //SDカードライブラリを始動、通信速度24Ｍｈｚ(CPUサイクル80Mhzの30％)
  f1 = SD.open(fname1, FILE_APPEND);  //ｆ１ファイルにfname1を定義して追記形式でオープン、（writeは1回上書き形式）
  if (bootCount == 1) {//bootCountが1の時だけ、項目名を記入する
  f1.println(F("date_time,Humid,Temp,soilTemp,soilmoist,SR,EC"));  //ｆ１、ｃｓｖ最上段に項目を記入
  }
    DateTime now = rtc.now();//現在時刻を取得しSDカードへ記入
    f1.print(now.year(), DEC);
    f1.print('/');
    f1.print(now.month(), DEC);
    f1.print('/');
    f1.print(now.day(), DEC);
    f1.print(" ");
    f1.print(now.hour(), DEC);
    f1.print(':');
    f1.print(now.minute(), DEC);
    f1.print(':');
    f1.print(now.second(), DEC);
    f1.print(',');

    f1.print(h);//AM2302の湿度の値を入力
    f1.print(',');
    f1.print(t);//AM2302の温度の値を入力
    f1.print(',');
  
    f1.print(soiltemp);//地温系ds18b20の測定温度を入力
    f1.print(',');
    
    f1.print(soilmoist);//土壌水分計10HSの出力値（mV）を入力
    f1.print(',');
    
    f1.print(SRmV);//日射計として使用したソーラーパネルの出力値（mV）を入力
    f1.print(',');
 
    f1.print(soilEC);//簡易ECセンサーの出力値を入力
    f1.println(',');

  f1.close();  //各ファイルを閉じる

digitalWrite(mosfet,LOW);//電源off
delay(1000);


//動作確認用シリアルモニタ表示
//より消費電力を減らすためにはシリアルモニタ表示を削除するとよい
///*
Serial.println("測定時刻は　");
    Serial.print(now.year(), DEC);
    Serial.print('/');
    Serial.print(now.month(), DEC);
    Serial.print('/');
    Serial.print(now.day(), DEC);
    Serial.print(" ");
    Serial.print(now.hour(), DEC);
    Serial.print(':');
    Serial.print(now.minute(), DEC);
    Serial.print(':');
    Serial.print(now.second(), DEC);
    Serial.println();

Serial.println("温湿度は　");
    Serial.print(h);//AM2302の湿度の値を入力
Serial.print("% ");
    Serial.print(t);//AM2302の温度の値を入力
Serial.println("℃");

Serial.println("地温は　");
    Serial.print(soiltemp);//地温系ds18b20の測定温度を入力
Serial.println("℃");

Serial.println("土壌水分は　");
    Serial.print(soilmoist);//土壌水分計10HSの出力値（mV）を入力
Serial.println("mV");

Serial.println("日射量は　");
    Serial.print(SRmV);//日射計として使用したソーラーパネルの出力値（mV）を入力
Serial.println("mV");

Serial.println("ECは　");
    Serial.println(soilEC);//簡易ECセンサーの出力値を入力


Serial.println("動作終了");
//*/

digitalWrite(mosfet,LOW);//電源off、本体のスリープ前に各センサーへの電力供給を止める

//ディープスリープ実行
  esp_sleep_enable_timer_wakeup(TIME_TO_SLEEP * uS_TO_S_FACTOR);
  esp_deep_sleep_start();//スリープモードへ移行
} 

void loop() {
}
